
%{
#include <err.h>
#include <stdio.h>
#include <string.h>

#include "kurvmatch.h"

extern int yylex(void);
void yyerror(char *);
char *strfix(char *s);

#define	MAXP 100
double vgary[MAXP], vpary[MAXP], ipary[MAXP], vg2ary[MAXP];
double scary[MAXP], icary[MAXP];
int np, sn;

%}

%token NAME
%token IDENT
%token DESCR
%token STRING
%token TMU
%token NUM
%token DIOD
%token TRIOD
%token PENTOD
%token HEPTOD
%token OKTOD
%token SCREEN
%token PARAM
%token TVCT
%token TYPE
%token TCCG
%token TCCP
%token TCGP
%token TRGI
%token TPLOTX
%token TPLOTY
%token TPLOTG
%token TWA
%token TEPS

%union {
	char *strp;
	double f;
}

%type <strp>	STRING IDENT
%type <f>	NUM

%%

start:		  start stmt { }
		| { }
		;

stmt:		  NAME STRING { d.name = strfix($2); }
		| DESCR STRING { d.descr = strfix($2); }
		| TMU NUM { d.MU = $2; }
		| TVCT NUM { d.VCT = $2; }
		| TCCG NUM { d.CCG = $2; }
		| TCCP NUM { d.CCP = $2; }
		| TCGP NUM { d.CGP = $2; }
		| TRGI NUM { d.RGI = $2; }
		| TPLOTX NUM NUM NUM { d.Xb = $2; d.Xi = $3; d.Xt = $4; }
		| TPLOTY NUM NUM NUM { d.Yb = $2; d.Yi = $3; d.Yt = $4; }
		| TPLOTG NUM NUM NUM { d.Gb = $2; d.Gi = $3; d.Gt = $4; }
		| TWA NUM { d.Wa = $2; }
		| TEPS NUM { d.EPS = $2; }
		| TYPE DIOD
		| TYPE TRIOD
		| TYPE PENTOD
		| TYPE HEPTOD
		| TYPE OKTOD
		| SCREEN PARAM scparm {
			d.Vs = scary;
			d.Is = icary;
			d.sn = sn;
		}
		| DIOD PARAM diparm {
			d.Vp = vpary;
			d.Ip = ipary;
			d.Vg = vgary;	/* XXX make later code happy */
			d.Vg2 = vg2ary;	/* XXX make later code happy */
			d.n = np;
			d.type = DIODE;
		}
		| TRIOD PARAM triparm {
			d.Vg = vgary;
			d.Vp = vpary;
			d.Ip = ipary;
			d.Vg2 = vg2ary;	/* XXX make later code happy */
			d.n = np;
			d.type = TRIODE;
		}
		| PENTOD PARAM tetparm {
			d.Vg = vgary;
			d.Vp = vpary;
			d.Ip = ipary;
			d.Vg2 = vg2ary;
			d.n = np;
			d.type = PENTODE;
		}
		;

scparm:		  scparm NUM NUM {
			if (sn == MAXP) {
				warnx("too many params, ignoring");
			} else {
				scary[sn] = $2, icary[sn] = $3;
				sn++;
			}
		}
		| { }
		;

diparm:		  diparm NUM NUM {
			if (np == MAXP) {
				warnx("too many params, ignoring");
			} else {
				vpary[np] = $2, ipary[np] = $3;
				np++;
			}
		}
		| { }
		;

triparm:	  triparm NUM NUM NUM {
			if (np == MAXP) {
				warnx("too many params, ignoring");
			} else {
				vpary[np] = $2, vgary[np] = $3, ipary[np] = $4;
				np++;
			}
		}
		| { }
		;

tetparm:	  tetparm NUM NUM NUM NUM {
			if (np == MAXP) {
				warnx("too many params, ignoring");
			} else {
				vpary[np] = $2, vgary[np] = $3, 
				    ipary[np] = $4, vg2ary[np] = $5;
				np++;
			}
		}
		| { }
		;

%%

char *
strfix(char *s)
{
	s = strdup(s);
	s++;
	s[strlen(s)-1] = 0;
	return s;
}

void
yyerror(char *s)
{
	extern int line;

	errx(1, "yyerror line %d: %s", line, s);
}
